Package: gbm
Version: 2.0-8
Date: 2013-01-17
Title: Generalized Boosted Regression Models
Author: Greg Ridgeway <gregridgeway@gmail.com> with contributions from
        others
Maintainer: Harry Southworth <harry.southworth@gmail.com>
Depends: R (>= 2.9.0), survival, lattice, splines
Suggests: RUnit
Description: This package implements extensions to Freund and
        Schapire's AdaBoost algorithm and Friedman's gradient boosting
        machine. Includes regression methods for least squares,
        absolute loss, t-distribution loss, quantile regression,
        logistic, multinomial logistic, Poisson, Cox proportional
        hazards partial likelihood, AdaBoost exponential loss,
        Huberized hinge loss, and Learning to Rank measures
        (LambdaMart).
License: GPL (>= 2)
URL: http://code.google.com/p/gradientboostedmodels/
Packaged: 2013-01-17 19:02:01 UTC; harry
Repository: CRAN
Date/Publication: 2013-01-18 10:51:22
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 10:17:08 UTC; unix
