Package: gbm
Version: 2.2.3
Title: Generalized Boosted Regression Models
Authors@R: c(
  person("Greg", "Ridgeway",
         email = "gridge@upenn.edu",
         role = c("aut", "cre"),
         comment = c(ORCID = "0000-0001-6911-0804")),
  person("Daniel", "Edwards",
         email = "unknown@unknown.com",
         role = "ctb"),
  person("Brian", "Kriegler",
         email = "bkriegler@econone.com",
         role = "ctb"),
  person("Stefan", "Schroedl",
         email = "stefan@atomwise.com",
         role = "ctb"),
  person("Harry", "Southworth",
         email = "harry@dataclarityconsulting.co.uk",
         role = "ctb"),
  person("Brandon", "Greenwell",
         email = "greenwell.brandon@gmail.com",
         role = "ctb",
         comment = c(ORCID = "0000-0002-8120-0084")),
  person("Bradley", "Boehmke",
         email = "bradleyboehmke@gmail.com",
         role = "ctb",
         comment = c(ORCID = "0000-0002-3611-8516")),
  person("Jay", "Cunningham",
         email = "james@notbadafterall.com",
         role = "ctb"),
  person("GBM", "Developers", 
         role = "aut", 
         comment = "https://github.com/gbm-developers")
  )
Depends: R (>= 2.9.0)
Imports: lattice, parallel, survival
Suggests: covr, gridExtra, knitr, pdp, RUnit, splines, tinytest, vip,
        viridis, MASS
Description: An implementation of extensions to Freund and Schapire's AdaBoost 
  algorithm and Friedman's gradient boosting machine. Includes regression 
  methods for least squares, absolute loss, t-distribution loss, quantile 
  regression, logistic, multinomial logistic, Poisson, Cox proportional hazards 
  partial likelihood, AdaBoost exponential loss, Huberized hinge loss, and 
  Learning to Rank measures (LambdaMart). Originally developed by Greg Ridgeway.
  Newer version available at github.com/gbm-developers/gbm3.
License: GPL (>= 2) | file LICENSE
URL: https://github.com/gbm-developers/gbm
BugReports: https://github.com/gbm-developers/gbm/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-01-22 16:42:41 UTC; greg_
Author: Greg Ridgeway [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6911-0804>),
  Daniel Edwards [ctb],
  Brian Kriegler [ctb],
  Stefan Schroedl [ctb],
  Harry Southworth [ctb],
  Brandon Greenwell [ctb] (ORCID:
    <https://orcid.org/0000-0002-8120-0084>),
  Bradley Boehmke [ctb] (ORCID: <https://orcid.org/0000-0002-3611-8516>),
  Jay Cunningham [ctb],
  GBM Developers [aut] (https://github.com/gbm-developers)
Maintainer: Greg Ridgeway <gridge@upenn.edu>
Repository: CRAN
Date/Publication: 2026-01-22 17:40:02 UTC
